From d2292169279eb6209a9b7fa21e152cd4c72d1e84 Mon Sep 17 00:00:00 2001
From: Daniel Bornkessel <daniel@soundcloud.com>
Date: Mon, 11 Apr 2011 14:14:54 +0200
Subject: [PATCH] make sure all vary headers are on one line 0.9.7

---
 src/http/ngx_http_header_filter_module.c |   44 +++++++++++++++++++++--------
 src/http/ngx_http_request.h              |    2 +
 src/http/ngx_http_upstream.c             |   14 +++++++++-
 3 files changed, 47 insertions(+), 13 deletions(-)

diff --git a/src/http/ngx_http_header_filter_module.c b/src/http/ngx_http_header_filter_module.c
index bccf636..cd8205c 100644
--- a/src/http/ngx_http_header_filter_module.c
+++ b/src/http/ngx_http_header_filter_module.c
@@ -169,6 +169,7 @@ ngx_http_header_filter(ngx_http_request_t *r)
     struct sockaddr_in6       *sin6;
 #endif
     u_char                     addr[NGX_SOCKADDR_STRLEN];
+    u_char                     gzip_vary;
 
     if (r->header_sent) {
         return NGX_OK;
@@ -401,16 +402,25 @@ ngx_http_header_filter(ngx_http_request_t *r)
         len += sizeof("Connection: closed" CRLF) - 1;
     }
 
+    gzip_vary = 0;
 #if (NGX_HTTP_GZIP)
-    if (r->gzip_vary) {
-        if (clcf->gzip_vary) {
-            len += sizeof("Vary: Accept-Encoding" CRLF) - 1;
-
-        } else {
-            r->gzip_vary = 0;
-        }
+    if (r->gzip_vary && clcf->gzip_vary) {
+      gzip_vary = 1;
     }
 #endif
+    if(gzip_vary || r->vary) {
+      len += sizeof("Vary: " CRLF); // the CRLF for the end of the string
+    }
+    if(gzip_vary) {
+      len += sizeof("Accept-Encoding") - 1;
+    }
+    if(gzip_vary && r->vary) {
+      len += sizeof(", ") - 1;
+    }
+    if(r->vary) {
+      len += r->headers_out.vary.len;
+    }
+
 
     part = &r->headers_out.headers.part;
     header = part->elts;
@@ -565,12 +575,22 @@ ngx_http_header_filter(ngx_http_request_t *r)
                              sizeof("Connection: close" CRLF) - 1);
     }
 
-#if (NGX_HTTP_GZIP)
-    if (r->gzip_vary) {
-        b->last = ngx_cpymem(b->last, "Vary: Accept-Encoding" CRLF,
-                             sizeof("Vary: Accept-Encoding" CRLF) - 1);
+
+    if(gzip_vary || r->vary) {
+      b->last = ngx_cpymem(b->last, "Vary: ", sizeof("Vary: ") - 1);
+    }
+    if(gzip_vary) {
+      b->last = ngx_cpymem(b->last, "Accept-Encoding", sizeof("Accept-Encoding") - 1);
+    }
+    if(gzip_vary && r->vary) {
+      b->last = ngx_cpymem(b->last, ", ", sizeof(", ") - 1);
+    }
+    if(r->vary) {
+      b->last = ngx_cpymem(b->last, r->headers_out.vary.data, r->headers_out.vary.len);
+    }
+    if(gzip_vary || r->vary) {
+      b->last = ngx_cpymem(b->last, CRLF, sizeof(CRLF) - 1);
     }
-#endif
 
     part = &r->headers_out.headers.part;
     header = part->elts;
diff --git a/src/http/ngx_http_request.h b/src/http/ngx_http_request.h
index c20810e..d73e49a 100644
--- a/src/http/ngx_http_request.h
+++ b/src/http/ngx_http_request.h
@@ -254,6 +254,7 @@ typedef struct {
 
     size_t                            content_type_len;
     ngx_str_t                         content_type;
+    ngx_str_t                         vary;
     ngx_str_t                         charset;
     u_char                           *content_type_lowcase;
     ngx_uint_t                        content_type_hash;
@@ -471,6 +472,7 @@ struct ngx_http_request_s {
     unsigned                          gzip_ok:1;
     unsigned                          gzip_vary:1;
 #endif
+    unsigned                          vary:1;
 
     unsigned                          proxy:1;
     unsigned                          bypass_cache:1;
diff --git a/src/http/ngx_http_upstream.c b/src/http/ngx_http_upstream.c
index dc2d0c0..d773afa 100644
--- a/src/http/ngx_http_upstream.c
+++ b/src/http/ngx_http_upstream.c
@@ -89,6 +89,8 @@ static ngx_int_t ngx_http_upstream_process_buffering(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_charset(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
+static ngx_int_t ngx_http_upstream_copy_vary_line(ngx_http_request_t *r,
+    ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_copy_header_line(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t
@@ -246,7 +248,10 @@ ngx_http_upstream_header_t  ngx_http_upstream_headers_in[] = {
     { ngx_string("X-Accel-Charset"),
                  ngx_http_upstream_process_charset, 0,
                  ngx_http_upstream_copy_header_line, 0, 0 },
-
+    { ngx_string("Vary"),
+                 ngx_http_upstream_process_buffering, 0,
+                 ngx_http_upstream_copy_vary_line, 0, 0 },
+ 
 #if (NGX_HTTP_GZIP)
     { ngx_string("Content-Encoding"),
                  ngx_http_upstream_process_header_line,
@@ -3316,6 +3321,13 @@ ngx_http_upstream_process_charset(ngx_http_request_t *r, ngx_table_elt_t *h,
     return NGX_OK;
 }
 
+static ngx_int_t
+ngx_http_upstream_copy_vary_line(ngx_http_request_t *r, ngx_table_elt_t *h,
+    ngx_uint_t offset) {
+  r->headers_out.vary = h->value;
+  r->vary = 1;
+  return NGX_OK;
+}
 
 static ngx_int_t
 ngx_http_upstream_copy_header_line(ngx_http_request_t *r, ngx_table_elt_t *h,
-- 
1.7.3.5

